/*
 * Copyright (C) 2001-2006 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.producer;

/**
 * <p>A NodedProducer is the root of a tree of ProducerNodes.</p>
 * 
 * <p>There  used to be  a time  when there  were Producers  that weren't
 * NodedProducers, however they have been phased out.</p>
 */

public class NodedProducer implements Producer {
  private ProducerNode rootNode;
  private boolean isAborted;
  private boolean isFinished;
  protected final static String PRODUCER_KEY = "$producer";

  public NodedProducer(ProducerNode aRootNode) {
    rootNode = aRootNode;
    isAborted = false;
  }

  public boolean execute(ProductionContext aProductionContext) throws ProducerFailure, ProducerExc {
    rootNode.produce(aProductionContext);

    synchronized(this) {
      isFinished=true;

      return !isAborted;
    }
  }

  public void abort(ProductionContext aContext) {
    synchronized(this) {
      if (!isFinished)
        isAborted=true;
    }
  }

  public boolean getIsAborted() {
    synchronized(this) {
      return isAborted;
    }
  }
}
