/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mircoders.entity;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;

import mir.entity.AbstractEntity;
import mir.log.LoggerWrapper;
import mir.storage.DatabaseFailure;
import mircoders.storage.DatabaseContentToMedia;

/**
 * this class implements mapping of one line of the database table content
 * to a java object
 *
 * @version $Id: EntityContent.java,v 1.19.2.12 2005/10/30 00:46:58 zapata Exp $
 * @author mir-coders group
 *
 */


public class EntityContent extends AbstractEntity {
  /**
   * set is_produced flag for the article
   */
  public void setProduced(boolean yesno) throws DatabaseFailure
  {
    String value = (yesno) ? "1":"0";
    if (value.equals( getFieldValue("is_produced") )) return;

    Connection con=null;Statement stmt=null;
    String sql = "update content set is_produced='" + value + "' where id='" + getId()+"'";
    try {
      con = database.obtainConnection();
      /** todo should be preparedStatement: faster!! */
      stmt = con.createStatement();
      database.executeUpdate(stmt,sql);
    }
    catch (DatabaseFailure e) {
      throw e;
    }
    catch (SQLException e) {
      throw new DatabaseFailure(e);
    }
    finally {
      database.freeConnection(con,stmt);
    }
  }

  /**
   * Deattaches media from an article
   *
   * @param anArticleId
   * @param aMediaId
   * @throws DatabaseFailure
   */
  public void dettach(String anArticleId, String aMediaId) throws DatabaseFailure
  {
    if (aMediaId!=null){
      DatabaseContentToMedia.getInstance().delete(anArticleId, aMediaId);

      setProduced(false);
    }
  }

  /**
   * Attaches media to the article
   */

  public void attach(String aMediaId) throws DatabaseFailure
  {
    if (aMediaId!=null) {
      DatabaseContentToMedia.getInstance().addMedia(getId(),aMediaId);
      setProduced(false);
    }
    else {
      getLogger().error("EntityContent: attach without mid");
    }
  }

  /**
   * overridden method setFieldValues to patch creator_main_url
   */
  public void setFieldValues(Map theStringValues) {
    if (theStringValues != null) {
      if (theStringValues.containsKey("creator_main_url")){
        if (((String)theStringValues.get("creator_main_url")).equalsIgnoreCase("http://")){
          theStringValues.remove("creator_main_url");
        }
        else if (!((String)theStringValues.get("creator_main_url")).startsWith("http://")){
          theStringValues.put("creator_main_url","http://"+((String)theStringValues.get("creator_main_url")));
        }
      }
    }
    super.setFieldValues(theStringValues);
  }
}
