/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License,
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library
 * (or with modified versions of the above that use the same license as the above),
 * and distribute linked combinations including the two.  You must obey the
 * GNU General Public License in all respects for all of the code used other than
 * the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mircoders.localizer.basic;

import java.util.Collections;
import java.util.List;

import mir.util.HTMLRoutines;
import mir.util.JDBCStringRoutines;
import mir.util.ParameterExpander;
import mir.util.RewindableIterator;
import mir.util.StringRoutines;
import mir.util.StructuredContentParser;
import mir.util.SubsetIterator;

public class MirBasicUtilityFunctions {
  public String encodeXML(Object anObject) throws Exception {
    return HTMLRoutines.encodeXML(StringRoutines.interpretAsString(anObject));
  }

  public String encodeHTML(Object aString) throws Exception {
    return HTMLRoutines.encodeHTML(StringRoutines.interpretAsString(aString));
  }

  public String prettyEncodeHTML(Object aString) throws Exception {
    return HTMLRoutines.prettyEncodeHTML(StringRoutines.interpretAsString(aString));
  }

  public String encodeURI(Object aString) throws Exception {
    return HTMLRoutines.encodeURL(StringRoutines.interpretAsString(aString));
  }

  public String encodeURI(Object aString, Object anEncoding) throws Exception {
    return HTMLRoutines.encodeURL(
        StringRoutines.interpretAsString(aString),
        StringRoutines.interpretAsString(anEncoding));
  }

  public String subString(Object aString, Object aFrom) throws Exception {
    return StringRoutines.interpretAsString(aString).substring(StringRoutines.interpretAsInteger(aFrom));
  }

  public String subString(Object aString, Object aFrom, Object aLength) throws Exception {
    int length = StringRoutines.interpretAsInteger(aLength);
    String target = StringRoutines.interpretAsString(aString);
    if (length<0 || length>target.length()) {
      length=target.length();
    }

    return target.substring(StringRoutines.interpretAsInteger(aFrom), length);
  }

  public String escapeJDBCString(Object aString) throws Exception {
    return JDBCStringRoutines.escapeStringLiteral(StringRoutines.interpretAsString(aString));
  }

  public String constructString(Object aString) throws Exception {
    if (aString==null)
      return StructuredContentParser.constructStringLiteral("");
		return StructuredContentParser.constructStringLiteral(StringRoutines.interpretAsString(aString));
  }

  public Object parseStructuredString(Object aString) throws Exception {
    if (aString==null)
      return null;
		return StructuredContentParser.parse(StringRoutines.interpretAsString(aString));
  }

  public boolean isOdd(Object anInteger) throws Exception {
    return (StringRoutines.interpretAsInteger(anInteger) & 1) == 1;
  }

  public int increment(Object anInteger) throws Exception {
    final Integer ONE = new Integer(1);

    return increment(anInteger, ONE);
  }

  public int increment(Object anInteger, Object anIncrement) throws Exception {
    return StringRoutines.interpretAsInteger(anInteger) +
           StringRoutines.interpretAsInteger(anIncrement);
  }

  public Object subList(Object aList, Object aSkip) throws Exception {
    return subList(aList, aSkip, new Integer(-1));
  }

  public Object subList(Object aList, Object aSkip, Object aMaxSize) throws Exception {
    int skip = StringRoutines.interpretAsInteger(aSkip);
    int maxSize = StringRoutines.interpretAsInteger(aMaxSize);

    if (aList instanceof RewindableIterator)
      return new SubsetIterator((RewindableIterator) aList, skip, maxSize);
		
    List list = (List) aList;
		if (skip>=list.size())
		  return Collections.EMPTY_LIST;
		if (maxSize<0 || (skip+maxSize)>=list.size())
		  return list.subList(skip, list.size());
		return list.subList(skip, skip+maxSize);
  }

  public int listSize(RewindableIterator anIterator) {
    anIterator.rewind();
    int result=0;

    while (anIterator.hasNext()) {
      result++;
      anIterator.next();
    }

    anIterator.rewind();

    return result;
  }

  public int listSize(List aList) {
    return aList.size();
  }

  public Object evaluate(Object aTarget, String anExpression) throws Exception {
    return ParameterExpander.expandExpression(aTarget, anExpression);
  }

  public String regexpreplace(String aString, String anExpression, String aReplacement) {
    return StringRoutines.performRegularExpressionReplacement(aString, anExpression, aReplacement);
  }

  public boolean regexpmatch(String aString, String anExpression) {
    return StringRoutines.performRegularExpressionSearch(aString, anExpression);
  }
}
